/*
 * This file is part of the Owl Positioning System (OwlPS) project.
 * It is subject to the copyright notice and license terms in the
 * COPYRIGHT.t2t file found in the top-level directory of this
 * distribution and at
 *   https://code.lm7.fr/mcy/owlps/src/master/COPYRIGHT.t2t
 * No part of the OwlPS Project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms
 * contained in the COPYRIGHT.t2t file; the COPYRIGHT.t2t file must be
 * distributed along with this file, either separately or by replacing
 * this notice by the COPYRIGHT.t2t file's contents.
 */


#ifndef _OWLPS_POSITIONING_OUTPUTUDPSOCKET_HH_
#define _OWLPS_POSITIONING_OUTPUTUDPSOCKET_HH_

#include "outputnetworksocket.hh"

/// Sends results to a remote host by UDP
/**
 * The results are sent through an UDP socket as a CSV string value.
 * The format used is the same as in OutputCSV, and is documented in
 * ResultList::to_csv().
 */
class OutputUDPSocket: public OutputNetworkSocket
{
protected:
  /** @name Operations */
  //@{
  /// Initialises the socket
  bool init_socket(void) ;
  /// Sends a string through the socket
  bool send_data(const std::string &data) const ;
  //@}

public:
  OutputUDPSocket(const std::string &_remote_host,
                  const uint_fast16_t _remote_port) ;
} ;



#endif // _OWLPS_POSITIONING_OUTPUTUDPSOCKET_HH_
