/*
 * This file is part of the Owl Positioning System (OwlPS) project.
 * It is subject to the copyright notice and license terms in the
 * COPYRIGHT.t2t file found in the top-level directory of this
 * distribution and at
 *   https://code.lm7.fr/mcy/owlps/src/master/COPYRIGHT.t2t
 * No part of the OwlPS Project, including this file, may be copied,
 * modified, propagated, or distributed except according to the terms
 * contained in the COPYRIGHT.t2t file; the COPYRIGHT.t2t file must be
 * distributed along with this file, either separately or by replacing
 * this notice by the COPYRIGHT.t2t file's contents.
 */


#ifndef _OWLPS_POSITIONING_POSITIONING_HH_
#define _OWLPS_POSITIONING_POSITIONING_HH_

class PositioningAlgorithm ;
class Request ;
class Result ;

#include "input.hh"
#include "output.hh"

#include <vector>

/// Computes [results](@ref Result) from [requests](@ref Request) using
/// one or more [positioning algorithms](@ref PositioningAlgorithm)
class Positioning
{
protected:
  Input input ;
  Output output ;
  std::vector<PositioningAlgorithm*> algorithms ;

  void initialise_algorithms(void) ;
  void loop(void) ;
  /// Sets a new filtered position in `result`
  void filter(const Request &request, Result &result) ;

public:
  Positioning(void) ;
  ~Positioning(void) ;
} ;

#endif // _OWLPS_POSITIONING_POSITIONING_HH_
